"""add is clients isolated flag to projects

Revision ID: 4095103c7d01
Revises: 05ac7e8caa21
Create Date: 2022-06-22 15:09:56.585117

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "4095103c7d01"
down_revision = "05ac7e8caa21"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "project",
        sa.Column("is_clients_isolated", sa.Boolean(), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("project", "is_clients_isolated")
    # ### end Alembic commands ###
