"""For SearchFilterGroup.is_shared

Revision ID: 680c64565f9d
Revises: f344b867a911
Create Date: 2024-05-29 02:13:20.764614

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "680c64565f9d"
down_revision = "f344b867a911"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("search_filter_group", schema=None) as batch_op:
        batch_op.add_column(
            sa.Column("is_shared", sa.Boolean(), nullable=True)
        )

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("search_filter_group", schema=None) as batch_op:
        batch_op.drop_column("is_shared")

    # ### end Alembic commands ###
