"""Change file_size to big integer

Revision ID: df9f8a147e80
Revises: 92bdfe07e5f5
Create Date: 2022-03-14 18:01:49.812243

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "df9f8a147e80"
down_revision = "92bdfe07e5f5"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "preview_file",
        "file_size",
        existing_type=sa.INTEGER(),
        type_=sa.BigInteger(),
        existing_nullable=True,
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column(
        "preview_file",
        "file_size",
        existing_type=sa.BigInteger(),
        type_=sa.INTEGER(),
        existing_nullable=True,
    )
    # ### end Alembic commands ###
