"""discord_integration

Revision ID: 92bdfe07e5f5
Revises: aa0a60033106
Create Date: 2022-03-03 17:04:48.006864

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "92bdfe07e5f5"
down_revision = "aa0a60033106"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "organisation",
        sa.Column("chat_token_discord", sa.String(length=80), nullable=True),
    )
    op.add_column(
        "person",
        sa.Column(
            "notifications_discord_enabled", sa.Boolean(), nullable=True
        ),
    )
    op.add_column(
        "person",
        sa.Column(
            "notifications_discord_userid", sa.String(length=60), nullable=True
        ),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("person", "notifications_discord_userid")
    op.drop_column("person", "notifications_discord_enabled")
    op.drop_column("organisation", "chat_token_discord")
    # ### end Alembic commands ###
