"""empty message

Revision ID: 40dea9555940
Revises: 2adc020885fa
Create Date: 2019-05-09 18:19:12.766386

"""

from alembic import op


# revision identifiers, used by Alembic.
revision = "40dea9555940"
down_revision = "2adc020885fa"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("news_preview_file_id_fkey", "news", type_="foreignkey")
    op.create_foreign_key(
        None, "news", "preview_file", ["preview_file_id"], ["id"]
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "news", type_="foreignkey")
    op.create_foreign_key(
        "news_preview_file_id_fkey",
        "news",
        "task",
        ["preview_file_id"],
        ["id"],
    )
    # ### end Alembic commands ###
