"""drop task_status.is_reviewable

Revision ID: b8c0a0f9d054
Revises: 6eeaff945706
Create Date: 2022-04-29 15:04:36.498586

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "b8c0a0f9d054"
down_revision = "6eeaff945706"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task_status", "is_reviewable")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "task_status",
        sa.Column(
            "is_reviewable", sa.BOOLEAN(), autoincrement=False, nullable=True
        ),
    )
    # ### end Alembic commands ###
